@@ -10,7 +10,7 @@
 """
 The cyipopt_solver module includes the python interface to the
 Cythonized ipopt solver cyipopt (see more:
-https://github.com/matthias-k/cyipopt.git). To use the solver,
+https://github.com/mechmotum/cyipopt.git). To use the solver,
 you can create a derived implementation from the abstract base class
 CyIpoptProblemInterface that provides the necessary methods.
 
@@ -19,7 +19,6 @@ that works with problems derived from AslNLP as long as those
 classes return numpy ndarray objects for the vectors and coo_matrix
 objects for the matrices (e.g., AmplNLP and PyomoNLP)
 """
-import six
 import sys
 import logging
 import os
@@ -29,16 +28,37 @@ from pyomo.common.dependencies import (
     attempt_import,
     numpy as np, numpy_available,
 )
-ipopt, ipopt_available = attempt_import(
-    'ipopt',
-    error_message='cyipopt solver relies on the ipopt module from cyipopt. '
-    'See https://github.com/matthias-k/cyipopt.git for cyipopt '
-    'installation instructions.'
+
+def _cyipopt_importer():
+    import cyipopt
+    # cyipopt before version 1.0.3 called the problem class "Problem"
+    if not hasattr(cyipopt, 'Problem'):
+        cyipopt.Problem = cyipopt.problem
+    # cyipopt before version 1.0.3 put the __version__ flag in the ipopt
+    # module (which was deprecated starting in 1.0.3)
+    if not hasattr(cyipopt, '__version__'):
+        import ipopt
+        cyipopt.__version__ = ipopt.__version__
+    # Beginning in 1.0.3, STATUS_MESSAGES is in a separate
+    # ipopt_wrapper module
+    if not hasattr(cyipopt, 'STATUS_MESSAGES'):
+        import ipopt_wrapper
+        cyipopt.STATUS_MESSAGES = ipopt_wrapper.STATUS_MESSAGES
+    return cyipopt
+
+cyipopt, cyipopt_available = attempt_import(
+     'ipopt',
+     error_message='cyipopt solver relies on the ipopt module from cyipopt. '
+     'See https://github.com/mechmotum/cyipopt.git for cyipopt '
+     'installation instructions.',
+     importer=_cyipopt_importer,
 )
+
 # Because pynumero.interfaces requires numpy, we will leverage deferred
 # imports here so that the solver can be registered even when numpy is
 # not available.
 pyomo_nlp = attempt_import('pyomo.contrib.pynumero.interfaces.pyomo_nlp')[0]
+pyomo_grey_box = attempt_import('pyomo.contrib.pynumero.interfaces.pyomo_grey_box_nlp')[0]
 egb = attempt_import('pyomo.contrib.pynumero.interfaces.external_grey_box')[0]
 
 from pyomo.common.config import ConfigBlock, ConfigValue
@@ -53,25 +73,48 @@ logger = logging.getLogger(__name__)
 # This maps the cyipopt STATUS_MESSAGES back to string representations
 # of the Ipopt ApplicationReturnStatus enum
 _cyipopt_status_enum = [
-    'Solve_Succeeded', b'Algorithm terminated successfully at a locally optimal point, satisfying the convergence tolerances (can be specified by options).',
-    'Solved_To_Acceptable_Level', b'Algorithm stopped at a point that was converged, not to "desired" tolerances, but to "acceptable" tolerances (see the acceptable-... options).',
-    'Infeasible_Problem_Detected', b'Algorithm converged to a point of local infeasibility. Problem may be infeasible.',
-    'Search_Direction_Becomes_Too_Small', b'Algorithm proceeds with very little progress.',
-    'Diverging_Iterates', b'It seems that the iterates diverge.',
-    'User_Requested_Stop', b'The user call-back function intermediate_callback (see Section 3.3.4 in the documentation) returned false, i.e., the user code requested a premature termination of the optimization.',
-    'Feasible_Point_Found', b'Feasible point for square problem found.',
-    'Maximum_Iterations_Exceeded', b'Maximum number of iterations exceeded (can be specified by an option).',
-    'Restoration_Failed', b'Restoration phase failed, algorithm doesn\'t know how to proceed.',
-    'Error_In_Step_Computation', b'An unrecoverable error occurred while Ipopt tried to compute the search direction.',
-    'Maximum_CpuTime_Exceeded', b'Maximum CPU time exceeded.',
-    'Not_Enough_Degrees_Of_Freedom', b'Problem has too few degrees of freedom.',
-    'Invalid_Problem_Definition', b'Invalid problem definition.',
-    'Invalid_Option', b'Invalid option encountered.',
-    'Invalid_Number_Detected', b'Algorithm received an invalid number (such as NaN or Inf) from the NLP; see also option check_derivatives_for_naninf',
-    'Unrecoverable_Exception', b'Some uncaught Ipopt exception encountered.',
-    'NonIpopt_Exception_Thrown', b'Unknown Exception caught in Ipopt',
-    'Insufficient_Memory', b'Not enough memory.',
-    'Internal_Error', b'An unknown internal error occurred. Please contact the Ipopt authors through the mailing list.'
+    "Solve_Succeeded", (b"Algorithm terminated successfully at a locally "
+                        b"optimal point, satisfying the convergence tolerances "
+                        b"(can be specified by options)."),
+    "Solved_To_Acceptable_Level", (b"Algorithm stopped at a point that was "
+                                   b"converged, not to \"desired\" tolerances, "
+                                   b"but to \"acceptable\" tolerances (see the "
+                                   b"acceptable-... options)."),
+    "Infeasible_Problem_Detected", (b"Algorithm converged to a point of local "
+                                    b"infeasibility. Problem may be "
+                                    b"infeasible."),
+    "Search_Direction_Becomes_Too_Small", (b"Algorithm proceeds with very "
+                                           b"little progress."),
+    "Diverging_Iterates", b"It seems that the iterates diverge.",
+    "User_Requested_Stop", (b"The user call-back function intermediate_callback "
+                            b"(see Section 3.3.4 in the documentation) returned "
+                            b"false, i.e., the user code requested a premature "
+                            b"termination of the optimization."),
+    "Feasible_Point_Found", b"Feasible point for square problem found.",
+    "Maximum_Iterations_Exceeded", (b"Maximum number of iterations exceeded "
+                                    b"(can be specified by an option)."),
+    "Restoration_Failed", (b"Restoration phase failed, algorithm doesn\'t know "
+                           b"how to proceed."),
+    "Error_In_Step_Computation", (b"An unrecoverable error occurred while Ipopt "
+                                  b"tried to compute the search direction."),
+    "Maximum_CpuTime_Exceeded", b"Maximum CPU time exceeded.",
+    "Not_Enough_Degrees_Of_Freedom", b"Problem has too few degrees of freedom.",
+    "Invalid_Problem_Definition", b"Invalid problem definition.",
+    "Invalid_Option", b"Invalid option encountered.",
+    "Invalid_Number_Detected", (b"Algorithm received an invalid number (such as "
+                                b"NaN or Inf) from the NLP; see also option "
+                                b"check_derivatives_for_naninf."),
+    # Note that the concluding "." was missing before cyipopt 1.0.3
+    "Invalid_Number_Detected", (b"Algorithm received an invalid number (such as "
+                                b"NaN or Inf) from the NLP; see also option "
+                                b"check_derivatives_for_naninf"),
+    "Unrecoverable_Exception", b"Some uncaught Ipopt exception encountered.",
+    "NonIpopt_Exception_Thrown", b"Unknown Exception caught in Ipopt.",
+    # Note that the concluding "." was missing before cyipopt 1.0.3
+    "NonIpopt_Exception_Thrown", b"Unknown Exception caught in Ipopt",
+    "Insufficient_Memory", b"Not enough memory.",
+    "Internal_Error", (b"An unknown internal error occurred. Please contact "
+                       b"the Ipopt authors through the mailing list."),
 ]
 _cyipopt_status_enum = {
     _cyipopt_status_enum[i+1]: _cyipopt_status_enum[i]
@@ -102,8 +145,7 @@ _ipopt_term_cond = {
     'Internal_Error': TerminationCondition.internalSolverError,
 }
 
-@six.add_metaclass(abc.ABCMeta)
-class CyIpoptProblemInterface(object):
+class CyIpoptProblemInterface(object, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def x_init(self):
         """Return the initial values for x as a numpy ndarray
@@ -213,7 +255,7 @@ class CyIpoptProblemInterface(object):
 
 
 class CyIpoptNLP(CyIpoptProblemInterface):
-    def __init__(self, nlp):
+    def __init__(self, nlp, intermediate_callback=None):
         """This class provides a CyIpoptProblemInterface for use
         with the CyIpoptSolver class that can take in an NLP
         as long as it provides vectors as numpy ndarrays and
@@ -222,6 +264,8 @@ class CyIpoptNLP(CyIpoptProblemInterface):
         and the CyIpoptSolver
         """
         self._nlp = nlp
+        self._intermediate_callback = intermediate_callback
+
         x = nlp.init_primals()
         y = nlp.init_duals()
         if np.any(np.isnan(y)):
@@ -241,7 +285,7 @@ class CyIpoptNLP(CyIpoptProblemInterface):
             self._hess_lag = nlp.evaluate_hessian_lag()
             self._hess_lower_mask = self._hess_lag.row >= self._hess_lag.col
             self._hessian_available = True
-        except NotImplementedError:
+        except (AttributeError, NotImplementedError):
             self._hessian_available = False
             self._hess_lag = None
             self._hess_lower_mask = None
@@ -336,7 +380,11 @@ class CyIpoptNLP(CyIpoptProblemInterface):
             alpha_pr,
             ls_trials
     ):
-        pass
+        if self._intermediate_callback is not None:
+            return self._intermediate_callback(self._nlp, alg_mod, iter_count, obj_value,
+                                               inf_pr, inf_du, mu, d_norm, regularization_size,
+                                               alpha_du, alpha_pr, ls_trials)
+        return True
 
 
 def _redirect_stdout():
@@ -391,7 +439,7 @@ class CyIpoptSolver(object):
         nx = len(xstart)
         ng = len(gl)
 
-        cyipopt_solver = ipopt.problem(
+        cyipopt_solver = cyipopt.Problem(
             n=nx,
             m=ng,
             problem_obj=self._problem,
@@ -448,8 +496,18 @@ class PyomoCyIpoptSolver(object):
         domain=bool,
         description="Store the final solution into the original Pyomo model",
     ))
+    CONFIG.declare("return_nlp", ConfigValue(
+        default=False,
+        domain=bool,
+        description="Return the results object and the underlying nlp"
+                    " NLP object from the solve call.",
+    ))
     CONFIG.declare("options", ConfigBlock(implicit=True))
-
+    CONFIG.declare("intermediate_callback", ConfigValue(
+        default=None,
+        description="Set the function that will be called each"
+                    " iteration."
+    ))
 
     def __init__(self, **kwds):
         """Create an instance of the CyIpoptSolver. You must
@@ -465,13 +523,13 @@ class PyomoCyIpoptSolver(object):
         self._model = model
 
     def available(self, exception_flag=False):
-        return numpy_available and ipopt_available
+        return numpy_available and cyipopt_available
 
     def license_is_valid(self):
         return True
 
     def version(self):
-        return tuple(int(_) for _ in ipopt.__version__.split('.'))
+        return tuple(int(_) for _ in cyipopt.__version__.split('.'))
 
     def solve(self, model, **kwds):
         config = self.config(kwds, preserve_implicit=True)
@@ -485,10 +543,12 @@ class PyomoCyIpoptSolver(object):
         grey_box_blocks = list(model.component_data_objects(
             egb.ExternalGreyBoxBlock, active=True))
         if grey_box_blocks:
-            nlp = pyomo_nlp.PyomoGreyBoxNLP(model)
+            # nlp = pyomo_nlp.PyomoGreyBoxNLP(model)
+            nlp = pyomo_grey_box.PyomoNLPWithGreyBoxBlocks(model)
         else:
             nlp = pyomo_nlp.PyomoNLP(model)
-        problem = CyIpoptNLP(nlp)
+
+        problem = CyIpoptNLP(nlp, intermediate_callback=config.intermediate_callback)
 
         xl = problem.x_lb()
         xu = problem.x_ub()
@@ -498,7 +558,7 @@ class PyomoCyIpoptSolver(object):
         nx = len(xl)
         ng = len(gl)
 
-        cyipopt_solver = ipopt.problem(
+        cyipopt_solver = cyipopt.Problem(
             n=nx,
             m=ng,
             problem_obj=problem,
@@ -538,7 +598,8 @@ class PyomoCyIpoptSolver(object):
             logger.error(msg, exc_info=sys.exc_info())
             solverStatus = SolverStatus.unknown
             raise
-        wall_time = timer.toc("")
+
+        wall_time = timer.toc(None)
 
         results = SolverResults()
 
@@ -585,6 +646,10 @@ class PyomoCyIpoptSolver(object):
         results.solver.termination_condition = _ipopt_term_cond[status_enum]
         results.solver.status = TerminationCondition.to_solver_status(
             results.solver.termination_condition)
+
+        if config.return_nlp:
+            return results, nlp
+
         return results
 
     #
