@@ -21,10 +21,13 @@ import logging
 from weakref import ref as weakref_ref
 import inspect
 
+from pyomo.common.log import is_debug_set
+from pyomo.common.deprecation import deprecated, RenamedClass
 from pyomo.common.timing import ConstructionTimer
 from pyomo.core.expr.numvalue import value
-from pyomo.core.base.plugin import ModelComponentFactory
-from pyomo.core.base.component import ActiveComponentData
+from pyomo.core.base.component import (
+    ActiveComponentData, ModelComponentFactory,
+)
 from pyomo.core.base.indexed_component import (ActiveIndexedComponent,
                                                UnindexedComponent_set,
                                                _get_indexed_component_data_name)
@@ -34,8 +37,6 @@ from pyomo.core.base.misc import apply_indexed_rule, tabular_writer
 from pyomo.core.base.set import Set
 from pyomo.core.base import minimize, maximize
 
-from six import iteritems
-
 logger = logging.getLogger('pyomo.core')
 
 _rule_returned_none_error = """Objective '%s': rule returned None.
@@ -280,7 +281,7 @@ class Objective(ActiveIndexedComponent):
         if cls != Objective:
             return super(Objective, cls).__new__(cls)
         if not args or (args[0] is UnindexedComponent_set and len(args)==1):
-            return SimpleObjective.__new__(SimpleObjective)
+            return ScalarObjective.__new__(ScalarObjective)
         else:
             return IndexedObjective.__new__(IndexedObjective)
 
@@ -319,7 +320,7 @@ class Objective(ActiveIndexedComponent):
         """
         Construct the expression(s) for this objective.
         """
-        if __debug__ and logger.isEnabledFor(logging.DEBUG):
+        if is_debug_set(logger):
             logger.debug(
                 "Constructing objective %s" % (self.name))
         if self._constructed:
@@ -330,13 +331,6 @@ class Objective(ActiveIndexedComponent):
         _init_expr = self._init_expr
         _init_sense = self._init_sense
         _init_rule = self.rule
-        #
-        # We no longer need these
-        #
-        self._init_expr = None
-        self._init_sense = None
-        # Utilities like DAE assume this stays around
-        #self.rule = None
 
         if (_init_rule is None) and \
            (_init_expr is None):
@@ -400,7 +394,7 @@ class Objective(ActiveIndexedComponent):
              ("Index", self._index if self.is_indexed() else None),
              ("Active", self.active)
              ],
-            iteritems(self._data),
+            self._data.items(),
             ( "Active","Sense","Expression"),
             lambda k, v: [ v.active,
                            ("minimize" if (v.sense == minimize) else "maximize"),
@@ -424,7 +418,7 @@ class Objective(ActiveIndexedComponent):
 
         ostream.write("\n")
         tabular_writer( ostream, prefix+tab,
-                        ((k,v) for k,v in iteritems(self._data) if v.active),
+                        ((k,v) for k,v in self._data.items() if v.active),
                         ( "Active","Value" ),
                         lambda k, v: [ v.active, value(v), ] )
 
@@ -454,9 +448,9 @@ class Objective(ActiveIndexedComponent):
 
         return expr
 
-class SimpleObjective(_GeneralObjectiveData, Objective):
+class ScalarObjective(_GeneralObjectiveData, Objective):
     """
-    SimpleObjective is the implementation representing a single,
+    ScalarObjective is the implementation representing a single,
     non-indexed objective.
     """
 
@@ -485,7 +479,7 @@ class SimpleObjective(_GeneralObjectiveData, Objective):
         if self._constructed:
             if len(self._data) == 0:
                 raise ValueError(
-                    "Accessing the expression of SimpleObjective "
+                    "Accessing the expression of ScalarObjective "
                     "'%s' before the Objective has been assigned "
                     "a sense or expression. There is currently "
                     "nothing to access." % (self.name))
@@ -502,16 +496,15 @@ class SimpleObjective(_GeneralObjectiveData, Objective):
 
     # for backwards compatibility reasons
     @property
+    @deprecated("The .value property getter on ScalarObjective is deprecated. "
+                "Use the .expr property getter instead", version='4.3.11323')
     def value(self):
-        logger.warning("DEPRECATED: The .value property getter on "
-                       "SimpleObjective is deprecated. Use "
-                       "the .expr property getter instead")
         return self.expr
+
     @value.setter
+    @deprecated("The .value property setter on ScalarObjective is deprecated. "
+                "Use the set_value(expr) method instead", version='4.3.11323')
     def value(self, expr):
-        logger.warning("DEPRECATED: The .value property setter on "
-                       "SimpleObjective is deprecated. Use the "
-                       "set_value(expr) method instead")
         self.set_value(expr)
 
     @property
@@ -520,7 +513,7 @@ class SimpleObjective(_GeneralObjectiveData, Objective):
         if self._constructed:
             if len(self._data) == 0:
                 raise ValueError(
-                    "Accessing the sense of SimpleObjective "
+                    "Accessing the sense of ScalarObjective "
                     "'%s' before the Objective has been assigned "
                     "a sense or expression. There is currently "
                     "nothing to access." % (self.name))
@@ -546,6 +539,9 @@ class SimpleObjective(_GeneralObjectiveData, Objective):
     # Objective.Skip but expects a valid expression or None
     #
 
+    def clear(self):
+        self._data = {}
+
     def set_value(self, expr):
         """Set the expression of this objective."""
         if not self._constructed:
@@ -582,12 +578,18 @@ class SimpleObjective(_GeneralObjectiveData, Objective):
         """Add an expression with a given index."""
         if index is not None:
             raise ValueError(
-                "SimpleObjective object '%s' does not accept "
+                "ScalarObjective object '%s' does not accept "
                 "index values other than None. Invalid value: %s"
                 % (self.name, index))
         self.set_value(expr)
         return self
 
+
+class SimpleObjective(metaclass=RenamedClass):
+    __renamed__new_class__ = ScalarObjective
+    __renamed__version__ = '6.0'
+
+
 class IndexedObjective(Objective):
 
     #
@@ -624,8 +626,7 @@ class ObjectiveList(IndexedObjective):
         """
         Construct the expression(s) for this objective.
         """
-        generate_debug_messages = \
-            __debug__ and logger.isEnabledFor(logging.DEBUG)
+        generate_debug_messages = is_debug_set(logger)
         if generate_debug_messages:
             logger.debug(
                 "Constructing objective %s" % (self.name))
