@@ -14,9 +14,9 @@ Unit Tests for pyomo.dae.misc
 import os
 from os.path import abspath, dirname
 
-from six import StringIO, iterkeys
+from io import StringIO
 
-import pyutilib.th as unittest
+import pyomo.common.unittest as unittest
 
 from pyomo.environ import (
     ConcreteModel, Set, Param, Var, Constraint, Expression, Block,
@@ -484,7 +484,7 @@ class TestDaeMisc(unittest.TestCase):
 
         generate_finite_elements(model.t, 5)
 
-        missing_idx = set(model.blk._index) - set(iterkeys(model.blk._data))
+        missing_idx = set(model.blk._index) - set(model.blk._data.keys())
         model.blk._dae_missing_idx = missing_idx
 
         update_contset_indexed_component(model.blk, expansion_map)
@@ -537,7 +537,7 @@ class TestDaeMisc(unittest.TestCase):
 
         generate_finite_elements(model.t, 5)
 
-        missing_idx = set(model.blk._index) - set(iterkeys(model.blk._data))
+        missing_idx = set(model.blk._index) - set(model.blk._data.keys())
         model.blk._dae_missing_idx = missing_idx
 
         update_contset_indexed_component(model.blk, expansion_map)
@@ -547,8 +547,8 @@ class TestDaeMisc(unittest.TestCase):
         self.assertEqual(len(model.blk[2, 'B'].con1), 18)
         self.assertEqual(len(model.blk[10, 'C'].v2), 4)
 
-        self.assertEqual(model.blk[2, 'A'].p1['A', 2], 4)
-        self.assertEqual(model.blk[8, 'C'].p1['B', 6], 12)
+        self.assertEqual(model.blk[2, 'A'].p1['A', 2].value, 4)
+        self.assertEqual(model.blk[8, 'C'].p1['B', 6].value, 12)
         
         self.assertEqual(model.blk[4, 'B'].con1['B', 4](), 15)
         self.assertEqual(model.blk[6, 'A'].con1['C', 8](), 55)
@@ -592,7 +592,7 @@ class TestDaeMisc(unittest.TestCase):
 
         generate_finite_elements(model.t, 5)
 
-        missing_idx = set(model.blk._index) - set(iterkeys(model.blk._data))
+        missing_idx = set(model.blk._index) - set(model.blk._data.keys())
         model.blk._dae_missing_idx = missing_idx
 
         update_contset_indexed_component(model.blk, expansion_map)
@@ -647,7 +647,7 @@ class TestDaeMisc(unittest.TestCase):
 
         generate_finite_elements(model.t, 5)
 
-        missing_idx = set(model.blk._index) - set(iterkeys(model.blk._data))
+        missing_idx = set(model.blk._index) - set(model.blk._data.keys())
         model.blk._dae_missing_idx = missing_idx
 
         update_contset_indexed_component(model.blk, expansion_map)
@@ -657,8 +657,8 @@ class TestDaeMisc(unittest.TestCase):
         self.assertEqual(len(model.blk[2, 'B'].con1), 18)
         self.assertEqual(len(model.blk[10, 'C'].v2), 4)
 
-        self.assertEqual(model.blk[2, 'A'].p1['A', 2], 4)
-        self.assertEqual(model.blk[8, 'C'].p1['B', 6], 12)
+        self.assertEqual(model.blk[2, 'A'].p1['A', 2].value, 4)
+        self.assertEqual(model.blk[8, 'C'].p1['B', 6].value, 12)
         
         self.assertEqual(model.blk[4, 'B'].con1['B', 4](), 15)
         self.assertEqual(model.blk[6, 'A'].con1['C', 8](), 55)
