@@ -11,14 +11,16 @@
 # Unit Tests for DataPortal objects
 #
 
+from itertools import zip_longest
+import json
 import os
-from os.path import abspath, dirname
+from os.path import abspath, dirname, join
 pyomo_dir=dirname(dirname(abspath(__file__)))+os.sep+".."
 
-import pyutilib.misc
-import pyutilib.th as unittest
+import pyomo.common.unittest as unittest
 
 from pyomo.common.errors import ApplicationError
+from pyomo.common.tee import capture_output
 from pyomo.dataportal.factory import DataManagerFactory
 from pyomo.environ import AbstractModel, ConcreteModel, Set, DataPortal, Param, Boolean, Any, value
 
@@ -44,6 +46,7 @@ except:
     xlsm_interface = False
 try:
     yaml_interface = DataManagerFactory('yaml').available()
+    import yaml
 except:
     yaml_interface = False
 
@@ -670,18 +673,36 @@ class TestOnlyJsonPortal(TestOnlyTextPortal):
     def create_options(self, name):
         return {'filename':os.path.abspath(tutorial_dir+os.sep+'json'+os.sep+name+self.suffix)}
 
+    def compare_data(self, name, file_suffix):
+        if file_suffix == '.json':
+            with open(join(currdir, name+file_suffix), 'r') as out, \
+                open(join(currdir, name+'.baseline'+file_suffix), 'r') as txt:
+                self.assertStructuredAlmostEqual(json.load(txt), json.load(out),
+                                                 allow_second_superset=True,
+                                                 abstol=0)
+        elif file_suffix == '.yaml':
+            with open(join(currdir, name+file_suffix), 'r') as out, \
+                open(join(currdir, name+'.baseline'+file_suffix), 'r') as txt:
+                self.assertStructuredAlmostEqual(yaml.full_load(txt),
+                                                 yaml.full_load(out),
+                                                 allow_second_superset=True,
+                                                 abstol=0)
+        else:
+            with open(join(currdir, name+file_suffix), 'r') as f1, \
+                open(join(currdir, name+'.baseline'+file_suffix), 'r') as f2:
+                    f1_contents = list(filter(None, f1.read().split()))
+                    f2_contents = list(filter(None, f2.read().split()))
+                    for item1, item2 in zip_longest(f1_contents, f2_contents):
+                        self.assertEqual(item1, item2)
+        os.remove(currdir+name+file_suffix)
+
     def test_store_set1(self):
         # Write 1-D set
         model = ConcreteModel()
         model.A = Set(initialize=set([1,3,5]))
         data = DataPortal()
         data.store(data=model.A, **self.create_write_options('set1'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
+        self.compare_data('set1', self.suffix)
 
     def test_store_set1a(self):
         # Write 1-D set
@@ -689,12 +710,7 @@ class TestOnlyJsonPortal(TestOnlyTextPortal):
         model.A = Set(initialize=set([1,3,5]))
         data = DataPortal()
         data.store(data="A", model=model, **self.create_write_options('set1'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
+        self.compare_data('set1', self.suffix)
 
     def test_store_set2(self):
         # Write 2-D set
@@ -702,12 +718,7 @@ class TestOnlyJsonPortal(TestOnlyTextPortal):
         model.A = Set(initialize=set([(1,2),(3,4),(5,6)]), dimen=2)
         data = DataPortal()
         data.store(data=model.A, **self.create_write_options('set2'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'set2'+self.suffix, currdir+'set2.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'set2'+self.suffix, currdir+'set2.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'set2'+self.suffix, currdir+'set2.baseline'+self.suffix)
+        self.compare_data('set2', self.suffix)
 
     def test_store_param1(self):
         # Write scalar param
@@ -715,12 +726,7 @@ class TestOnlyJsonPortal(TestOnlyTextPortal):
         model.p = Param(initialize=1)
         data = DataPortal()
         data.store(data=model.p, **self.create_write_options('param1'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param1'+self.suffix, currdir+'param1.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param1'+self.suffix, currdir+'param1.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param1'+self.suffix, currdir+'param1.baseline'+self.suffix)
+        self.compare_data('param1', self.suffix)
 
     def test_store_param2(self):
         # Write 1-D param
@@ -729,12 +735,7 @@ class TestOnlyJsonPortal(TestOnlyTextPortal):
         model.p = Param(model.A, initialize={1:10, 2:20, 3:30})
         data = DataPortal()
         data.store(data=model.p, **self.create_write_options('param2'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param2'+self.suffix, currdir+'param2.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param2'+self.suffix, currdir+'param2.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param2'+self.suffix, currdir+'param2.baseline'+self.suffix)
+        self.compare_data('param2', self.suffix)
 
     def test_store_param3(self):
         # Write 2-D params
@@ -744,12 +745,7 @@ class TestOnlyJsonPortal(TestOnlyTextPortal):
         model.q = Param(model.A, initialize={(1,2):11, (2,3):21, (3,4):31})
         data = DataPortal()
         data.store(data=(model.p,model.q), **self.create_write_options('param3'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param3'+self.suffix, currdir+'param3.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param3'+self.suffix, currdir+'param3.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param3'+self.suffix, currdir+'param3.baseline'+self.suffix)
+        self.compare_data('param3', self.suffix)
 
     def test_store_param4(self):
         # Write 2-D params
@@ -759,12 +755,7 @@ class TestOnlyJsonPortal(TestOnlyTextPortal):
         model.q = Param(model.A, initialize={(1,2):11, (2,3):21, (3,4):31})
         data = DataPortal()
         data.store(data=(model.p,model.q), columns=('a','b','c','d'), **self.create_write_options('param4'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param4'+self.suffix, currdir+'param4.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param4'+self.suffix, currdir+'param4.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param4'+self.suffix, currdir+'param4.baseline'+self.suffix)
+        self.compare_data('param4', self.suffix)
 
 
 @unittest.skipIf(not yaml_interface, "No YAML interface available")
@@ -791,6 +782,35 @@ class TestTextPortal(unittest.TestCase):
     def create_write_options(self, name):
         return {'filename':os.path.abspath(currdir+os.sep+name+self.suffix), 'sort':True}
 
+    def compare_data(self, name, file_suffix):
+        if file_suffix == '.json':
+            with open(join(currdir, name+file_suffix), 'r') as out, \
+                open(join(currdir, name+'.baseline'+file_suffix), 'r') as txt:
+                self.assertStructuredAlmostEqual(json.load(txt), json.load(out),
+                                                 allow_second_superset=True,
+                                                 abstol=0)
+        elif file_suffix == '.yaml':
+            with open(join(currdir, name+file_suffix), 'r') as out, \
+                open(join(currdir, name+'.baseline'+file_suffix), 'r') as txt:
+                self.assertStructuredAlmostEqual(yaml.full_load(txt),
+                                                 yaml.full_load(out),
+                                                 allow_second_superset=True,
+                                                 abstol=0)
+        else:
+            try:
+                with open(join(currdir, name+file_suffix), 'r') as f1, \
+                open(join(currdir, name+'.baseline'+file_suffix), 'r') as f2:
+                    f1_contents = list(filter(None, f1.read().split()))
+                    f2_contents = list(filter(None, f2.read().split()))
+                    for item1, item2 in zip_longest(f1_contents, f2_contents):
+                        self.assertEqual(item1, item2)
+            except:
+                with open(join(currdir, name+file_suffix), 'r') as out, \
+                    open(join(currdir, name+'.baseline'+file_suffix), 'r') as txt:
+                    self.assertEqual(out.read().strip().replace(' ',''),
+                                     txt.read().strip().replace(' ',''))
+        os.remove(currdir+name+file_suffix)
+
     def test_tableA(self):
         # Importing an unordered set of arbitrary data
         self.check_skiplist('tableA')
@@ -971,12 +991,7 @@ class TestTextPortal(unittest.TestCase):
         model.A = Set(initialize=set([1,3,5]))
         data = DataPortal()
         data.store(set=model.A, **self.create_write_options('set1'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'set1'+self.suffix, currdir+'set1.baseline'+self.suffix)
+        self.compare_data('set1', self.suffix)
 
     def test_store_set2(self):
         # Write 2-D set
@@ -985,12 +1000,7 @@ class TestTextPortal(unittest.TestCase):
         model.A = Set(initialize=set([(1,2),(3,4),(5,6)]), dimen=2)
         data = DataPortal()
         data.store(set=model.A, **self.create_write_options('set2'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'set2'+self.suffix, currdir+'set2.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'set2'+self.suffix, currdir+'set2.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'set2'+self.suffix, currdir+'set2.baseline'+self.suffix)
+        self.compare_data('set2', self.suffix)
 
     def test_store_param1(self):
         # Write scalar param
@@ -999,12 +1009,7 @@ class TestTextPortal(unittest.TestCase):
         model.p = Param(initialize=1)
         data = DataPortal()
         data.store(param=model.p, **self.create_write_options('param1'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param1'+self.suffix, currdir+'param1.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param1'+self.suffix, currdir+'param1.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param1'+self.suffix, currdir+'param1.baseline'+self.suffix)
+        self.compare_data('param1', self.suffix)
 
     def test_store_param2(self):
         # Write 1-D param
@@ -1014,12 +1019,7 @@ class TestTextPortal(unittest.TestCase):
         model.p = Param(model.A, initialize={1:10, 2:20, 3:30})
         data = DataPortal()
         data.store(param=model.p, **self.create_write_options('param2'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param2'+self.suffix, currdir+'param2.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param2'+self.suffix, currdir+'param2.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param2'+self.suffix, currdir+'param2.baseline'+self.suffix)
+        self.compare_data('param2', self.suffix)
 
     def test_store_param3(self):
         # Write 2-D params
@@ -1030,12 +1030,7 @@ class TestTextPortal(unittest.TestCase):
         model.q = Param(model.A, initialize={(1,2):11, (2,3):21, (3,4):31})
         data = DataPortal()
         data.store(param=(model.p,model.q), **self.create_write_options('param3'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param3'+self.suffix, currdir+'param3.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param3'+self.suffix, currdir+'param3.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param3'+self.suffix, currdir+'param3.baseline'+self.suffix)
+        self.compare_data('param3', self.suffix)
 
     def test_store_param4(self):
         # Write 2-D params
@@ -1046,12 +1041,7 @@ class TestTextPortal(unittest.TestCase):
         model.q = Param(model.A, initialize={(1,2):11, (2,3):21, (3,4):31})
         data = DataPortal()
         data.store(param=(model.p,model.q), **self.create_write_options('param4'))
-        if self.suffix == '.json':
-            self.assertMatchesJsonBaseline(currdir+'param4'+self.suffix, currdir+'param4.baseline'+self.suffix)
-        elif self.suffix == '.yaml':
-            self.assertMatchesYamlBaseline(currdir+'param4'+self.suffix, currdir+'param4.baseline'+self.suffix)
-        else:
-            self.assertFileEqualsBaseline(currdir+'param4'+self.suffix, currdir+'param4.baseline'+self.suffix)
+        self.compare_data('param4', self.suffix)
 
 
 class TestCsvPortal(TestTextPortal):
@@ -1116,9 +1106,8 @@ class LoadTests(object):
     def test_tableA1(self):
         # Importing a single column of data
         self.check_skiplist('tableA1')
-        pyutilib.misc.setup_redirect(currdir+'loadA1.dat')
-        print("load "+self.filename('A')+" format=set : A;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadA1.dat'):
+            print("load "+self.filename('A')+" format=set : A;")
         model=AbstractModel()
         model.A = Set()
         instance = model.create_instance(currdir+'loadA1.dat')
@@ -1128,9 +1117,8 @@ class LoadTests(object):
     def test_tableA2(self):
         # Importing a single column of data
         self.check_skiplist('tableA2')
-        pyutilib.misc.setup_redirect(currdir+'loadA2.dat')
-        print("load "+self.filename('A')+" ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadA2.dat'):
+            print("load "+self.filename('A')+" ;")
         model=AbstractModel()
         model.A = Set()
         try:
@@ -1145,9 +1133,8 @@ class LoadTests(object):
     def test_tableA3(self):
         # Importing a single column of data
         self.check_skiplist('tableA3')
-        pyutilib.misc.setup_redirect(currdir+'loadA3.dat')
-        print("load "+self.filename('A')+" format=set : A ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadA3.dat'):
+            print("load "+self.filename('A')+" format=set : A ;")
         model=AbstractModel()
         model.A = Set()
         instance = model.create_instance(currdir+'loadA3.dat')
@@ -1157,9 +1144,8 @@ class LoadTests(object):
     def test_tableB1(self):
         # Same as test_tableA
         self.check_skiplist('tableB1')
-        pyutilib.misc.setup_redirect(currdir+'loadB.dat')
-        print("load "+self.filename('B')+" format=set : B;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadB.dat'):
+            print("load "+self.filename('B')+" format=set : B;")
         model=AbstractModel()
         model.B = Set()
         instance = model.create_instance(currdir+'loadB.dat')
@@ -1170,9 +1156,8 @@ class LoadTests(object):
         # Importing a multi-column table, where all columns are
         # treated as values for a set with tuple values.
         self.check_skiplist('tableC')
-        pyutilib.misc.setup_redirect(currdir+'loadC.dat')
-        print("load "+self.filename('C')+" format=set : C ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadC.dat'):
+            print("load "+self.filename('C')+" format=set : C ;")
         model=AbstractModel()
         model.C = Set(dimen=2)
         instance = model.create_instance(currdir+'loadC.dat')
@@ -1182,9 +1167,8 @@ class LoadTests(object):
     def test_tableD(self):
         # Importing a 2D array of data as a set.
         self.check_skiplist('tableD')
-        pyutilib.misc.setup_redirect(currdir+'loadD.dat')
-        print("load "+self.filename('D')+" format=set_array : C ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadD.dat'):
+            print("load "+self.filename('D')+" format=set_array : C ;")
         model=AbstractModel()
         model.C = Set(dimen=2)
         instance = model.create_instance(currdir+'loadD.dat')
@@ -1194,9 +1178,8 @@ class LoadTests(object):
     def test_tableZ(self):
         # Importing a single parameter
         self.check_skiplist('tableZ')
-        pyutilib.misc.setup_redirect(currdir+'loadZ.dat')
-        print("load "+self.filename('Z')+" : Z ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadZ.dat'):
+            print("load "+self.filename('Z')+" : Z ;")
         model=AbstractModel()
         model.Z = Param(default=99.0)
         instance = model.create_instance(currdir+'loadZ.dat')
@@ -1206,9 +1189,8 @@ class LoadTests(object):
     def test_tableY(self):
         # Same as tableXW.
         self.check_skiplist('tableY')
-        pyutilib.misc.setup_redirect(currdir+'loadY.dat')
-        print("load "+self.filename('Y')+" : [A] Y;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadY.dat'):
+            print("load "+self.filename('Y')+" : [A] Y;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.Y = Param(model.A)
@@ -1222,9 +1204,8 @@ class LoadTests(object):
         # parameter columns.  The first column is assumed to represent an
         # index column.
         self.check_skiplist('tableXW_1')
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("load "+self.filename('XW')+" : [A] X W;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("load "+self.filename('XW')+" : [A] X W;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.X = Param(model.A)
@@ -1238,9 +1219,8 @@ class LoadTests(object):
     def test_tableXW_2(self):
         # Like test_tableXW_1, except that set A is not defined.
         self.check_skiplist('tableXW_2')
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("load "+self.filename('XW')+" : [A] X W;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("load "+self.filename('XW')+" : [A] X W;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3'])
         model.X = Param(model.A)
@@ -1253,9 +1233,8 @@ class LoadTests(object):
     def test_tableXW_3(self):
         # Like test_tableXW_1, except that set A is defined in the load statment.
         self.check_skiplist('tableXW_3')
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("load "+self.filename('XW')+" : A=[A] X W;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("load "+self.filename('XW')+" : A=[A] X W;")
         model=AbstractModel()
         model.A = Set()
         model.X = Param(model.A)
@@ -1269,9 +1248,8 @@ class LoadTests(object):
     def test_tableXW_4(self):
         # Like test_tableXW_1, except that set A is defined in the load statment and all values are mapped.
         self.check_skiplist('tableXW_4')
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("load "+self.filename('XW')+" : B=[A] R=X S=W;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("load "+self.filename('XW')+" : B=[A] R=X S=W;")
         model=AbstractModel()
         model.B = Set()
         model.R = Param(model.B)
@@ -1285,9 +1263,8 @@ class LoadTests(object):
     def test_tableT(self):
         # Importing a 2D array of parameters that are transposed.
         self.check_skiplist('tableT')
-        pyutilib.misc.setup_redirect(currdir+'loadT.dat')
-        print("load "+self.filename('T')+" format=transposed_array : T;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadT.dat'):
+            print("load "+self.filename('T')+" format=transposed_array : T;")
         model=AbstractModel()
         model.B = Set(initialize=['I1','I2','I3','I4'])
         model.A = Set(initialize=['A1','A2','A3'])
@@ -1299,9 +1276,8 @@ class LoadTests(object):
     def test_tableU(self):
         # Importing a 2D array of parameters.
         self.check_skiplist('tableU')
-        pyutilib.misc.setup_redirect(currdir+'loadU.dat')
-        print("load "+self.filename('U')+" format=array : U;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadU.dat'):
+            print("load "+self.filename('U')+" format=array : U;")
         model=AbstractModel()
         model.A = Set(initialize=['I1','I2','I3','I4'])
         model.B = Set(initialize=['A1','A2','A3'])
@@ -1315,9 +1291,8 @@ class LoadTests(object):
         # parameter columns.  The first column is assumed to represent an
         # index column.  A missing value is represented in the column data.
         self.check_skiplist('tableS')
-        pyutilib.misc.setup_redirect(currdir+'loadS.dat')
-        print("load "+self.filename('S')+" : [A] S ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadS.dat'):
+            print("load "+self.filename('S')+" : [A] S ;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.S = Param(model.A)
@@ -1329,9 +1304,8 @@ class LoadTests(object):
     def test_tablePO(self):
         # Importing a table that has multiple indexing columns
         self.check_skiplist('tablePO')
-        pyutilib.misc.setup_redirect(currdir+'loadPO.dat')
-        print("load "+self.filename('PO')+" : J=[A,B] P O;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadPO.dat'):
+            print("load "+self.filename('PO')+" : J=[A,B] P O;")
         model=AbstractModel()
         model.J = Set(dimen=2)
         model.P = Param(model.J)
@@ -1363,9 +1337,8 @@ class TestXmlLoad(LoadTests, unittest.TestCase):
         # parameter columns.  The first column is assumed to represent an
         # index column.
         self.check_skiplist('tableXW_1')
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("load "+self.filename('XW_nested1')+" query='./bar/table/*' : [A] X W;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("load "+self.filename('XW_nested1')+" query='./bar/table/*' : [A] X W;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.X = Param(model.A)
@@ -1381,9 +1354,8 @@ class TestXmlLoad(LoadTests, unittest.TestCase):
         # parameter columns.  The first column is assumed to represent an
         # index column.
         self.check_skiplist('tableXW_1')
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("load "+self.filename('XW_nested2')+" query='./bar/table/row' : [A] X W;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("load "+self.filename('XW_nested2')+" query='./bar/table/row' : [A] X W;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.X = Param(model.A)
@@ -1433,9 +1405,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableA1_1(self):
         # Importing a single column of data as a set
-        pyutilib.misc.setup_redirect(currdir+'loadA1.dat')
-        print("table columns=1 A={1} := A1 A2 A3 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadA1.dat'):
+            print("table columns=1 A={1} := A1 A2 A3 ;")
         model=AbstractModel()
         model.A = Set()
         instance = model.create_instance(currdir+'loadA1.dat')
@@ -1444,9 +1415,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableA1_2(self):
         # Importing a single column of data as a set
-        pyutilib.misc.setup_redirect(currdir+'loadA1.dat')
-        print("table A={A} : A := A1 A2 A3 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadA1.dat'):
+            print("table A={A} : A := A1 A2 A3 ;")
         model=AbstractModel()
         model.A = Set()
         instance = model.create_instance(currdir+'loadA1.dat')
@@ -1455,9 +1425,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableB1_1(self):
         # Same as test_tableA
-        pyutilib.misc.setup_redirect(currdir+'loadB.dat')
-        print("table columns=1 B={1} := 1 2 3 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadB.dat'):
+            print("table columns=1 B={1} := 1 2 3 ;")
         model=AbstractModel()
         model.B = Set()
         instance = model.create_instance(currdir+'loadB.dat')
@@ -1466,9 +1435,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableB1_2(self):
         # Same as test_tableA
-        pyutilib.misc.setup_redirect(currdir+'loadB.dat')
-        print("table B={B} : B := 1 2 3 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadB.dat'):
+            print("table B={B} : B := 1 2 3 ;")
         model=AbstractModel()
         model.B = Set()
         instance = model.create_instance(currdir+'loadB.dat')
@@ -1478,9 +1446,8 @@ class TestTableCmd(unittest.TestCase):
     def test_tableC_1(self):
         # Importing a multi-column table, where all columns are
         # treated as values for a set with tuple values.
-        pyutilib.misc.setup_redirect(currdir+'loadC.dat')
-        print("table columns=2 C={1,2} := A1 1 A1 2 A1 3 A2 1 A2 2 A2 3 A3 1 A3 2 A3 3 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadC.dat'):
+            print("table columns=2 C={1,2} := A1 1 A1 2 A1 3 A2 1 A2 2 A2 3 A3 1 A3 2 A3 3 ;")
         model=AbstractModel()
         model.C = Set(dimen=2)
         instance = model.create_instance(currdir+'loadC.dat')
@@ -1490,9 +1457,8 @@ class TestTableCmd(unittest.TestCase):
     def test_tableC_2(self):
         # Importing a multi-column table, where all columns are
         # treated as values for a set with tuple values.
-        pyutilib.misc.setup_redirect(currdir+'loadC.dat')
-        print("table C={a,b} : a b := A1 1 A1 2 A1 3 A2 1 A2 2 A2 3 A3 1 A3 2 A3 3 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadC.dat'):
+            print("table C={a,b} : a b := A1 1 A1 2 A1 3 A2 1 A2 2 A2 3 A3 1 A3 2 A3 3 ;")
         model=AbstractModel()
         model.C = Set(dimen=2)
         instance = model.create_instance(currdir+'loadC.dat')
@@ -1501,9 +1467,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableZ(self):
         # Importing a single parameter
-        pyutilib.misc.setup_redirect(currdir+'loadZ.dat')
-        print("table Z := 1.01 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadZ.dat'):
+            print("table Z := 1.01 ;")
         model=AbstractModel()
         model.Z = Param(default=99.0)
         instance = model.create_instance(currdir+'loadZ.dat')
@@ -1512,9 +1477,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableY_1(self):
         # Same as tableXW.
-        pyutilib.misc.setup_redirect(currdir+'loadY.dat')
-        print("table columns=2 Y(1)={2} := A1 3.3 A2 3.4 A3 3.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadY.dat'):
+            print("table columns=2 Y(1)={2} := A1 3.3 A2 3.4 A3 3.5 ;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.Y = Param(model.A)
@@ -1525,9 +1489,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableY_2(self):
         # Same as tableXW.
-        pyutilib.misc.setup_redirect(currdir+'loadY.dat')
-        print("table Y(A) : A Y := A1 3.3 A2 3.4 A3 3.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadY.dat'):
+            print("table Y(A) : A Y := A1 3.3 A2 3.4 A3 3.5 ;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.Y = Param(model.A)
@@ -1540,9 +1503,8 @@ class TestTableCmd(unittest.TestCase):
         # Importing a table, but only reporting the values for the non-index
         # parameter columns.  The first column is assumed to represent an
         # index column.
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("table columns=3 X(1)={2} W(1)={3} := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("table columns=3 X(1)={2} W(1)={3} := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.X = Param(model.A)
@@ -1557,9 +1519,8 @@ class TestTableCmd(unittest.TestCase):
         # Importing a table, but only reporting the values for the non-index
         # parameter columns.  The first column is assumed to represent an
         # index column.
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("table X(A) W(A) : A X W := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("table X(A) W(A) : A X W := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.X = Param(model.A)
@@ -1572,9 +1533,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableXW_3_1(self):
         # Like test_tableXW_1, except that set A is defined in the load statment.
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("table columns=3 A={1} X(A)={2} W(A)={3} := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("table columns=3 A={1} X(A)={2} W(A)={3} := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
         model=AbstractModel()
         model.A = Set()
         model.X = Param(model.A)
@@ -1587,9 +1547,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tableXW_3_2(self):
         # Like test_tableXW_1, except that set A is defined in the load statment.
-        pyutilib.misc.setup_redirect(currdir+'loadXW.dat')
-        print("table A={A} X(A) W(A) : A X W := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadXW.dat'):
+            print("table A={A} X(A) W(A) : A X W := A1 3.3 4.3 A2 3.4 4.4 A3 3.5 4.5 ;")
         model=AbstractModel()
         model.A = Set()
         model.X = Param(model.A)
@@ -1604,9 +1563,8 @@ class TestTableCmd(unittest.TestCase):
         # Importing a table, but only reporting the values for the non-index
         # parameter columns.  The first column is assumed to represent an
         # index column.  A missing value is represented in the column data.
-        pyutilib.misc.setup_redirect(currdir+'loadS.dat')
-        print("table columns=2 S(1)={2} := A1 3.3 A2 . A3 3.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadS.dat'):
+            print("table columns=2 S(1)={2} := A1 3.3 A2 . A3 3.5 ;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.S = Param(model.A)
@@ -1619,9 +1577,8 @@ class TestTableCmd(unittest.TestCase):
         # Importing a table, but only reporting the values for the non-index
         # parameter columns.  The first column is assumed to represent an
         # index column.  A missing value is represented in the column data.
-        pyutilib.misc.setup_redirect(currdir+'loadS.dat')
-        print("table S(A) : A S := A1 3.3 A2 . A3 3.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadS.dat'):
+            print("table S(A) : A S := A1 3.3 A2 . A3 3.5 ;")
         model=AbstractModel()
         model.A = Set(initialize=['A1','A2','A3','A4'])
         model.S = Param(model.A)
@@ -1632,9 +1589,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tablePO_1(self):
         # Importing a table that has multiple indexing columns
-        pyutilib.misc.setup_redirect(currdir+'loadPO.dat')
-        print("table columns=4 J={1,2} P(J)={3} O(J)={4} := A1 B1 4.3 5.3 A2 B2 4.4 5.4 A3 B3 4.5 5.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadPO.dat'):
+            print("table columns=4 J={1,2} P(J)={3} O(J)={4} := A1 B1 4.3 5.3 A2 B2 4.4 5.4 A3 B3 4.5 5.5 ;")
         model=AbstractModel()
         model.J = Set(dimen=2)
         model.P = Param(model.J)
@@ -1647,9 +1603,8 @@ class TestTableCmd(unittest.TestCase):
 
     def test_tablePO_2(self):
         # Importing a table that has multiple indexing columns
-        pyutilib.misc.setup_redirect(currdir+'loadPO.dat')
-        print("table J={A,B} P(J) O(J) : A B P O := A1 B1 4.3 5.3 A2 B2 4.4 5.4 A3 B3 4.5 5.5 ;")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadPO.dat'):
+            print("table J={A,B} P(J) O(J) : A B P O := A1 B1 4.3 5.3 A2 B2 4.4 5.4 A3 B3 4.5 5.5 ;")
         model=AbstractModel()
         model.J = Set(dimen=2)
         model.P = Param(model.J)
@@ -1662,13 +1617,12 @@ class TestTableCmd(unittest.TestCase):
 
     def test_complex_1(self):
         # Importing a table with multiple indexing columns
-        pyutilib.misc.setup_redirect(currdir+'loadComplex.dat')
-        print("table columns=8 I={4} J={3,5} A(I)={1} B(J)={7} :=")
-        print("A1 x1 J311 I1 J321 y1 B1 z1")
-        print("A2 x2 J312 I2 J322 y2 B2 z2")
-        print("A3 x3 J313 I3 J323 y3 B3 z3")
-        print(";")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadComplex.dat'):
+            print("table columns=8 I={4} J={3,5} A(I)={1} B(J)={7} :=")
+            print("A1 x1 J311 I1 J321 y1 B1 z1")
+            print("A2 x2 J312 I2 J322 y2 B2 z2")
+            print("A3 x3 J313 I3 J323 y3 B3 z3")
+            print(";")
         model=AbstractModel()
         model.I = Set()
         model.J = Set(dimen=2)
@@ -1683,14 +1637,13 @@ class TestTableCmd(unittest.TestCase):
 
     def test_complex_2(self):
         # Importing a table with multiple indexing columns
-        pyutilib.misc.setup_redirect(currdir+'loadComplex.dat')
-        print("table I={I} J={J1,J2} A(J) B(I) :")
-        print("A  x  J1   I  J2   y  B  z :=")
-        print("A1 x1 J311 I1 J321 y1 B1 z1")
-        print("A2 x2 J312 I2 J322 y2 B2 z2")
-        print("A3 x3 J313 I3 J323 y3 B3 z3")
-        print(";")
-        pyutilib.misc.reset_redirect()
+        with capture_output(currdir+'loadComplex.dat'):
+            print("table I={I} J={J1,J2} A(J) B(I) :")
+            print("A  x  J1   I  J2   y  B  z :=")
+            print("A1 x1 J311 I1 J321 y1 B1 z1")
+            print("A2 x2 J312 I2 J322 y2 B2 z2")
+            print("A3 x3 J313 I3 J323 y3 B3 z3")
+            print(";")
         model=AbstractModel()
         model.I = Set()
         model.J = Set(dimen=2)
