@@ -14,8 +14,9 @@ import logging
 
 from pyomo.common.collections import ComponentMap, ComponentSet
 from pyomo.common.config import ConfigBlock, ConfigValue
+from pyomo.common.log import is_debug_set
 from pyomo.common.modeling import unique_component_name
-from pyomo.common.deprecation import deprecated
+from pyomo.common.deprecation import deprecated, deprecation_warning
 from pyomo.contrib.fbbt.fbbt import compute_bounds_on_expr
 from pyomo.core import (
     Block, BooleanVar, Connector, Constraint, Param, Set, SetOf, Suffix, Var,
@@ -25,16 +26,15 @@ from pyomo.core.base.external import ExternalFunction
 from pyomo.core.base import Transformation, TransformationFactory, Reference
 import pyomo.core.expr.current as EXPR
 from pyomo.gdp import Disjunct, Disjunction, GDP_Error
-from pyomo.gdp.util import (
-    _warn_for_active_logical_constraint, target_list, is_child_of, get_src_disjunction,
-    get_src_constraint, get_transformed_constraints,
-    _get_constraint_transBlock, get_src_disjunct,
-    _warn_for_active_disjunction,
-    _warn_for_active_disjunct, )
+from pyomo.gdp.util import ( _warn_for_active_logical_constraint, target_list,
+                             is_child_of, get_src_disjunction,
+                             get_src_constraint, get_transformed_constraints,
+                             _get_constraint_transBlock, get_src_disjunct,
+                             _warn_for_active_disjunction,
+                             _warn_for_active_disjunct, preprocess_targets)
+from pyomo.network import Port
 from pyomo.repn import generate_standard_repn
-
 from functools import wraps
-from six import iterkeys, iteritems
 from weakref import ref as weakref_ref
 
 logger = logging.getLogger('pyomo.gdp.bigm')
@@ -161,11 +161,14 @@ class BigM_Transformation(Transformation):
             Block:       self._transform_block_on_disjunct,
             LogicalConstraint: self._warn_for_active_logical_statement,
             ExternalFunction: False,
+            Port:        False, # not Arcs, because those are deactivated after
+                                # the network.expand_arcs transformation
         }
+        self._generate_debug_messages = False
 
     def _get_bigm_suffix_list(self, block, stopping_block=None):
         # Note that you can only specify suffixes on BlockData objects or
-        # SimpleBlocks. Though it is possible at this point to stick them
+        # ScalarBlocks. Though it is possible at this point to stick them
         # on whatever components you want, we won't pick them up.
         suffix_list = []
 
@@ -196,6 +199,7 @@ class BigM_Transformation(Transformation):
 
     def _apply_to(self, instance, **kwds):
         assert not NAME_BUFFER
+        self._generate_debug_messages = is_debug_set(logger)
         self.used_args = ComponentMap() # If everything was sure to go well,
                                         # this could be a dictionary. But if
                                         # someone messes up and gives us a Var
@@ -211,6 +215,12 @@ class BigM_Transformation(Transformation):
             self.used_args.clear()
 
     def _apply_to_impl(self, instance, **kwds):
+        if not instance.ctype in (Block, Disjunct):
+            raise GDP_Error("Transformation called on %s of type %s. 'instance' "
+                            "must be a ConcreteModel, Block, or Disjunct (in "
+                            "the case of nested disjunctions)." %
+                            (instance.name, instance.ctype))
+
         config = self.CONFIG(kwds.pop('options', {}))
 
         # We will let args override suffixes and estimate as a last
@@ -218,8 +228,8 @@ class BigM_Transformation(Transformation):
         # the tree. Suffixes lower down in the tree override ones higher
         # up.
         if 'default_bigM' in kwds:
-            logger.warn("DEPRECATED: the 'default_bigM=' argument has been "
-                        "replaced by 'bigM='")
+            deprecation_warning("the 'default_bigM=' argument has been "
+                                "replaced by 'bigM='", version='5.4')
             config.bigM = kwds.pop('default_bigM')
 
         config.set_value(kwds)
@@ -229,9 +239,15 @@ class BigM_Transformation(Transformation):
         targets = config.targets
         if targets is None:
             targets = (instance, )
-        # We need to check that all the targets are in fact on instance. As we
-        # do this, we will use the set below to cache components we know to be
-        # in the tree rooted at instance.
+        else:
+            # we need to preprocess targets to make sure that if there are any
+            # disjunctions in targets that their disjuncts appear before them in
+            # the list.
+            targets = preprocess_targets(targets)
+
+        #  We need to check that all the targets are in fact on
+        # instance. As we do this, we will use the set below to cache components
+        # we know to be in the tree rooted at instance.
         knownBlocks = {}
         for t in targets:
             # check that t is in fact a child of instance
@@ -270,7 +286,7 @@ class BigM_Transformation(Transformation):
                         warning_msg += "\t%s\n" % component.name
                     else:
                         warning_msg += "\t%s\n" % component
-                logger.warn(warning_msg)
+                logger.warning(warning_msg)
 
     def _add_transformation_block(self, instance):
         # make a transformation block on instance to put transformed disjuncts
@@ -286,7 +302,7 @@ class BigM_Transformation(Transformation):
         return transBlock
 
     def _transform_block(self, obj, bigM):
-        for i in sorted(iterkeys(obj)):
+        for i in sorted(obj.keys()):
             self._transform_blockData(obj[i], bigM)
 
     def _transform_blockData(self, obj, bigM):
@@ -342,7 +358,7 @@ class BigM_Transformation(Transformation):
             transBlock = self._add_transformation_block(obj.parent_block())
 
         # relax each of the disjunctionDatas
-        for i in sorted(iterkeys(obj)):
+        for i in sorted(obj.keys()):
             self._transform_disjunctionData(obj[i], bigM, i, transBlock)
 
         # deactivate so the writers don't scream
@@ -378,7 +394,7 @@ class BigM_Transformation(Transformation):
                             obj.getname(fully_qualified=True,
                                         name_buffer=NAME_BUFFER))
         for disjunct in obj.disjuncts:
-            or_expr += disjunct.indicator_var
+            or_expr += disjunct.binary_indicator_var
             # make suffix list. (We don't need it until we are
             # transforming constraints, but it gets created at the
             # disjunct level, so more efficient to make it here and
@@ -405,7 +421,7 @@ class BigM_Transformation(Transformation):
         # deactivated -> either we've already transformed or user deactivated
         if not obj.active:
             if obj.indicator_var.is_fixed():
-                if value(obj.indicator_var) == 0:
+                if not value(obj.indicator_var):
                     # The user cleanly deactivated the disjunct: there
                     # is nothing for us to do here.
                     return
@@ -420,7 +436,7 @@ class BigM_Transformation(Transformation):
                     "indicator_var is not fixed and the disjunct does not "
                     "appear to have been relaxed. This makes no sense. "
                     "(If the intent is to deactivate the disjunct, fix its "
-                    "indicator_var to 0.)"
+                    "indicator_var to False.)"
                     % ( obj.name, ))
 
         if obj._transformation_block is not None:
@@ -465,21 +481,13 @@ class BigM_Transformation(Transformation):
 
     def _transform_block_components(self, block, disjunct, bigM, arg_list,
                                     suffix_list):
-        # Find all the variables declared here (including the indicator_var) and
-        # add a reference on the transformation block so these will be
-        # accessible when the Disjunct is deactivated. We don't descend into
-        # Disjuncts because we'll just reference the references which are
-        # already on their transformation blocks.
+        # We find any transformed disjunctions that might be here because we
+        # need to move their transformation blocks up onto the parent block
+        # before we transform anything else on this block. Note that we do this
+        # before we create references to local variables because we do not want
+        # duplicate references to indicator variables and local variables on
+        # nested disjuncts.
         disjunctBlock = disjunct._transformation_block()
-        varRefBlock = disjunctBlock.localVarReferences
-        for v in block.component_objects(Var, descend_into=Block, active=None):
-            varRefBlock.add_component(unique_component_name(
-                varRefBlock, v.getname(fully_qualified=True,
-                                       name_buffer=NAME_BUFFER)), Reference(v))
-
-        # Now need to find any transformed disjunctions that might be here
-        # because we need to move their transformation blocks up onto the parent
-        # block before we transform anything else on this block
         destinationBlock = disjunctBlock.parent_block()
         for obj in block.component_data_objects(
                 Disjunction,
@@ -497,6 +505,17 @@ class BigM_Transformation(Transformation):
             # we leave the transformation block because it still has the XOR
             # constraints, which we want to be on the parent disjunct.
 
+        # Find all the variables declared here (including the indicator_var) and
+        # add a reference on the transformation block so these will be
+        # accessible when the Disjunct is deactivated. We don't descend into
+        # Disjuncts because we just moved the references to their local
+        # variables up in the previous loop.
+        varRefBlock = disjunctBlock.localVarReferences
+        for v in block.component_objects(Var, descend_into=Block, active=None):
+            varRefBlock.add_component(unique_component_name(
+                varRefBlock, v.getname(fully_qualified=True,
+                                       name_buffer=NAME_BUFFER)), Reference(v))
+
         # Now look through the component map of block and transform everything
         # we have a handler for. Yell if we don't know how to handle it. (Note
         # that because we only iterate through active components, this means
@@ -523,7 +542,7 @@ class BigM_Transformation(Transformation):
         # we need to leave those on the disjunct.
         disjunctList = toBlock.relaxedDisjuncts
         to_delete = []
-        for idx, disjunctBlock in iteritems(fromBlock.relaxedDisjuncts):
+        for idx, disjunctBlock in fromBlock.relaxedDisjuncts.items():
             newblock = disjunctList[len(disjunctList)]
             newblock.transfer_attributes_from(disjunctBlock)
 
@@ -553,7 +572,8 @@ class BigM_Transformation(Transformation):
 
     def _warn_for_active_logical_statement(
             self, logical_statment, disjunct, infodict, bigMargs, suffix_list):
-        _warn_for_active_logical_constraint(logical_statment, disjunct, NAME_BUFFER)
+        _warn_for_active_logical_constraint(logical_statment, disjunct,
+                                            NAME_BUFFER)
 
     def _transform_block_on_disjunct(self, block, disjunct, bigMargs, arg_list,
                                      suffix_list):
@@ -562,7 +582,7 @@ class BigM_Transformation(Transformation):
         # directly.  (We are passing the disjunct through so that when
         # we find constraints, _xform_constraint will have access to
         # the correct indicator variable.)
-        for i in sorted(iterkeys(block)):
+        for i in sorted(block.keys()):
             self._transform_block_components( block[i], disjunct, bigMargs,
                                               arg_list, suffix_list)
 
@@ -612,7 +632,7 @@ class BigM_Transformation(Transformation):
             newConstraint = Constraint(obj.index_set(),
                                        disjunctionRelaxationBlock.lbub)
             # we map the container of the original to the container of the
-            # transformed constraint. Don't do this if obj is a SimpleConstraint
+            # transformed constraint. Don't do this if obj is a ScalarConstraint
             # because we will treat that like a _ConstraintData and map to a
             # list of transformed _ConstraintDatas
             constraintMap['transformedConstraints'][obj] = newConstraint
@@ -622,7 +642,7 @@ class BigM_Transformation(Transformation):
         # add mapping of transformed constraint to original constraint
         constraintMap['srcConstraints'][newConstraint] = obj
 
-        for i in sorted(iterkeys(obj)):
+        for i in sorted(obj.keys()):
             c = obj[i]
             if not c.active:
                 continue
@@ -636,7 +656,7 @@ class BigM_Transformation(Transformation):
                                                  upper)
             M = (lower[0], upper[0])
             
-            if __debug__ and logger.isEnabledFor(logging.DEBUG):
+            if self._generate_debug_messages:
                 _name = obj.getname(
                     fully_qualified=True, name_buffer=NAME_BUFFER)
                 logger.debug("GDP(BigM): The value for M for constraint '%s' "
@@ -655,7 +675,7 @@ class BigM_Transformation(Transformation):
                                                             lower, upper)
                 M = (lower[0], upper[0])
 
-            if __debug__ and logger.isEnabledFor(logging.DEBUG):
+            if self._generate_debug_messages:
                 _name = obj.getname(
                     fully_qualified=True, name_buffer=NAME_BUFFER)
                 logger.debug("GDP(BigM): The value for M for constraint '%s' "
@@ -669,7 +689,7 @@ class BigM_Transformation(Transformation):
                 M = (M[0], self._estimate_M(c.body, name)[1] - c.upper)
                 upper = (M[1], None, None)
 
-            if __debug__ and logger.isEnabledFor(logging.DEBUG):
+            if self._generate_debug_messages:
                 _name = obj.getname(
                     fully_qualified=True, name_buffer=NAME_BUFFER)
                 logger.debug("GDP(BigM): The value for M for constraint '%s' "
@@ -679,7 +699,7 @@ class BigM_Transformation(Transformation):
             # save the source information
             bigm_src[c] = (lower, upper)
 
-            # Handle indices for both SimpleConstraint and IndexedConstraint
+            # Handle indices for both ScalarConstraint and IndexedConstraint
             if i.__class__ is tuple:
                 i_lb = i + ('lb',)
                 i_ub = i + ('ub',)
@@ -694,7 +714,7 @@ class BigM_Transformation(Transformation):
                 if M[0] is None:
                     raise GDP_Error("Cannot relax disjunctive constraint '%s' "
                                     "because M is not defined." % name)
-                M_expr = M[0] * (1 - disjunct.indicator_var)
+                M_expr = M[0] * (1 - disjunct.binary_indicator_var)
                 newConstraint.add(i_lb, c.lower <= c. body - M_expr)
                 constraintMap[
                     'transformedConstraints'][c] = [newConstraint[i_lb]]
@@ -703,7 +723,7 @@ class BigM_Transformation(Transformation):
                 if M[1] is None:
                     raise GDP_Error("Cannot relax disjunctive constraint '%s' "
                                     "because M is not defined." % name)
-                M_expr = M[1] * (1 - disjunct.indicator_var)
+                M_expr = M[1] * (1 - disjunct.binary_indicator_var)
                 newConstraint.add(i_ub, c.body - M_expr <= c.upper)
                 transformed = constraintMap['transformedConstraints'].get(c)
                 if transformed is not None:
@@ -774,7 +794,7 @@ class BigM_Transformation(Transformation):
 
         # use the precomputed traversal up the blocks
         for arg in arg_list:
-            for block, val in iteritems(arg):
+            for block, val in arg.items():
                 (lower, upper, 
                  need_lower, need_upper) = self._process_M_value(val, lower,
                                                                  upper,
@@ -906,7 +926,7 @@ class BigM_Transformation(Transformation):
 
         # clean up if we unfixed things (fixed_vars is empty if we were assuming
         # fixed vars are fixed for life)
-        for v, val in iteritems(fixed_vars):
+        for v, val in fixed_vars.items():
             v.fix(val)
 
         return tuple(M)
